// main entry point

#include <stdlib.h>
#include <iostream>
#include <conio.h>
#include <string>

#include "segdwt.h"

#define OUTPUT_FILE "output.txt"

/* arguments:
	
	SIGNAL			/ filename, source signal samples
	FILTER			/ filename, source filter samples
	OUT_FILE		/ filename, output file name
	SEGMENT_SIZE	/ size of one segment 
	DEPTH			/ decomposition depth

*/


int main(int argc, char * argv[])
{
	// process arguments
	bool ok=false;
	bool argFail = false;
	bool debug = false;

	int wDepth, segmentSize;
	std::string inputFile;
	std::string filterFile;
	std::string outputFile;

	if(argc < 6) 
		argFail = true;
	else {
		// fetch segmentSize
		if(segmentSize = atoi(argv[4])) {
			if(segmentSize < 0) {
				std::cout << "Bad args: segmentSize must be positive" << std::endl;
				argFail = true;
			}
		} else {
			std::cout << "Bad args: segmentSize must be a non-zero positive integer" << std::endl;
			argFail = true;
		}
		// fetch wDepth
		if(wDepth = atoi(argv[5])) {
			if(wDepth < 0) {
				std::cout << "Bad args: wDepth must be positive" << std::endl;
				argFail = true;
			}
		} else {
			std::cout << "Bad args: wDepth must be a non-zero positive integer" << std::endl;
			argFail = true;
		}
	}

	if(argFail) {
		std::cout << std::endl << "Error in input arguments!" << std::endl;
		std::cout << "USAGE:" << std::endl;
		std::cout << "forward.exe signalFile filterFile outputFile segmentSize wtDepth" << std::endl;
		std::cout << "            filename   filename   filename   integer     integer" << std::endl;
		std::cout << " optional (at end): -debug" << std::endl;
		
		exit(0);
		
	} else {
		inputFile = argv[1];
		filterFile = argv[2];
		outputFile = argv[3];
	}

	if(argc > 6) {
		debug = true;			
	} 
	

	// perform forward segDWT
	
	try {
		// init transform
		SegDWT transform(segmentSize, wDepth, debug);
		
		// load the signal & filter
		transform.loadSignal(inputFile.c_str());
		transform.loadFilter(filterFile.c_str());
		
		// perform the transform
		transform.performTransform();
		
		// save result
		transform.saveResult(outputFile.c_str());
	}
	catch (...) {
		std::cout << "An exception occured. For solution, please contact the author with the exact description of input parameters. Terminating application..." << std::endl;
	}

	if(debug) {
		std::cout << "DEBUG OVER: press any key..." << std::endl;
		_getch();
	}
	
	return 0;
}

