// main entry point

#include <stdlib.h>
#include <iostream>
#include <conio.h>
#include <string>

#include "segidwt.h"

#define OUTPUT_FILE "signal-rec.txt"

/* arguments:
	
	COEF			/ filename, coef file generated by forward.exe
	FILTER			/ filename, source filter samples

*/


int main(int argc, char * argv[])
{
	// process arguments
	bool ok=false;
	bool argFail = false;
	bool debug = false;

	std::string inputFile;
	std::string filterFile;
	std::string outputFile;

	if(argc < 4) 
		argFail = true;

	if(argFail) {
		std::cout << std::endl << "Error in input arguments!" << std::endl;
		std::cout << "format: inverse.exe coefFile   filterFile  outputFile" << std::endl;
		std::cout << "                    filename   filename    filename" << std::endl;
		std::cout << "optional (at end): -debug" << std::endl;
		
		exit(0);
	} else {	
		inputFile = argv[1];
		filterFile = argv[2];
		outputFile = argv[3];
	}
	
	if(argc > 4) {
		debug = true;
	}
	

	// perform inverse segDWT
	
	try {
		// init transform
		SegIDWT transform(debug);
		
		// load the signal & filter
		transform.loadFilter(filterFile.c_str());
		transform.loadResult(inputFile.c_str());
		
		// perform the transform
		transform.performTransform();
		
		// save result
		transform.saveSignal(outputFile.c_str());
	}
	catch (...) {
		std::cout << "An exception occured. For solution, please contact the author with the exact description of input parameters. Terminating application.." << std::endl;
	}

	if(debug) {
		std::cout << "DEBUG OVER: press any key..." << std::endl;
		_getch();
	}

	return 0;
}


