// defines classes and declarations for SEGDWT
#ifndef SEGIDWT_H
#define SEGIDWT_H

#define DEBUG debug_

#include <iostream>
#include <vector>
#include <list>

// basic processing type declaration
typedef double type;

// filetype for vectors
typedef std::vector<type> vecType;
typedef vecType::iterator vecIter;

// std::cout overloads
using std::ostream;
ostream& operator<<(ostream& stream, vecType& vec);
ostream& operator<<(ostream& stream, std::vector<int>& vec);


class SegIDWT {
	// class declarations
	// segment is used for coefs storage
	struct Segment {
		struct WaveletCoefs {
			vecType a;
			vecType d;
		};
		bool					 isLast;		// if
		std::list<WaveletCoefs>	 coefs;			// contains result coefs. Scheme: {A D} {A D} {A D} ... 
		Segment(unsigned depth, bool lst);
	};	
	
	enum workType {dyadup, leftcut, rightcut};
	
	// weak params
	unsigned segCount_;		// count of segments, init after file load
	unsigned lastSegSize_;	// size of last segment
	unsigned n_;			// algorithm step
	unsigned segSize_;
	unsigned wDepth_;
	unsigned twoToJ_;
	unsigned extSize_;
	
	// for debug
	bool debug_;
	
	// storage
	vecType LoDFilter_;
	vecType LoRFilter_;
	vecType HiDFilter_;		// filters
	vecType HiRFilter_;
	
	vecType signal_;		// input signal
	
	std::list<Segment>	seg_;	// resulting segments
	
	// private methods
	void mlwr(SegIDWT::Segment &seg, vecType &src);
	void conv(vecType& signal, vecType& filter, vecType& result);
	void changeSignal(vecType& signal, workType type, unsigned cutOff=0);
	
	// private helpers (imported from common.h, not depending on class
	int parity(int last, int number);
	int GCD(int a, int b);
	int LCM(int a, int b);
	int GDPT(int n);
	int getPminVectSize(int J, int s);
	void genPminValues(std::vector<int>& pmin, int size, int s, int J);
		
public:
	// exceptions
	class ExcFilterFile {};
	class ExcSignalFile {};
	class ExcResultFile {};
	
	// PUBLIC INTERFACE
	// constructor
	SegIDWT(bool debg);
	// load signal
	void loadResult(const char *filename);
	// load filter
	void loadFilter(const char *filename);
	// process
	void performTransform();
	// save signal
	void saveSignal(const char *filename);
};

#endif